@@ -5,22 +5,38 @@ import ipywidgets as widgets
 import k3d
 import numpy as np
 from IPython.display import display
-from ipywidgets import Play, Checkbox, IntSlider, Button, ColorPicker, VBox, Tab
+from ipywidgets import Play, Checkbox, IntSlider, Button, ColorPicker, VBox, Tab, HBox, Accordion, Widget
 from k3d.plot import Plot
 
 from .jointset import JointSet, MuPoTSJoints, OpenPoseJoints, CocoExJoints, PanopticJoints, Common14Joints
 from .objects import Skeleton, DrawableSkeleton, Color, Positions
 
 
-class FrameUpdater:
-    def __init__(self, video_player: Play):
-        self.video_player: Play = video_player
+class VideoPlayer:
+    def __init__(self, plot: Plot):
+        self.plot: Plot = plot
+        self.video_player: Optional[Play] = None
 
-    def update_to_next_frame(self, button) -> None:
+    def get_video_player_widget(self, fps: int, frames: np.ndarray) -> HBox:
+        self.video_player = Play(
+            value=0,
+            min=0, max=frames.shape[0] - 1,
+            step=1, interval=1000 / fps,
+            description='Press play', disabled=False)
+        frame_slider = IntSlider(value=0, min=0, max=frames.shape[0] - 1, step=1, description='Frame')
+        next_frame_button = Button(description='Next frame')
+        next_frame_button.on_click(self.__update_to_next_frame)
+        previous_frame_button = Button(description='Previous frame')
+        previous_frame_button.on_click(self.__update_to_previous_frame)
+        widgets.jslink((self.video_player, 'value'), (frame_slider, 'value'))
+        widgets.jslink((self.video_player, 'value'), (self.plot, 'time'))
+        return HBox([self.video_player, previous_frame_button, frame_slider, next_frame_button])
+
+    def __update_to_next_frame(self, button) -> None:
         if self.video_player.max > self.video_player.value:
             self.video_player.value += 1
 
-    def update_to_previous_frame(self, button) -> None:
+    def __update_to_previous_frame(self, button) -> None:
         if self.video_player.min < self.video_player.value:
             self.video_player.value -= 1
 
@@ -59,28 +75,111 @@ class SkeletonColorUpdater:
 
 
 class ColorPickerSynchronizer:
-    def __init__(self, left_color_pickers: List[ColorPicker], right_color_pickers: List[ColorPicker],
-                 center_color_pickers: List[ColorPicker], all_color_pickers: List[ColorPicker]):
-        self.left_color_pickers: List[ColorPicker] = left_color_pickers
-        self.right_color_pickers: List[ColorPicker] = right_color_pickers
-        self.center_color_pickers: List[ColorPicker] = center_color_pickers
-        self.all_color_pickers: List[ColorPicker] = all_color_pickers
+    def __init__(self, left_skeleton_color_pickers: List[ColorPicker], right_skeleton_color_pickers: List[ColorPicker],
+                 center_skeleton_color_pickers: List[ColorPicker], all_skeleton_color_pickers: List[ColorPicker],
+                 left_synchronizer_color_picker: ColorPicker, right_synchronizer_color_picker: ColorPicker,
+                 center_synchronizer_color_picker: ColorPicker):
+        self.left_skeleton_color_pickers: List[ColorPicker] = left_skeleton_color_pickers
+        self.right_skeleton_color_pickers: List[ColorPicker] = right_skeleton_color_pickers
+        self.center_skeleton_color_pickers: List[ColorPicker] = center_skeleton_color_pickers
+        self.all_skeleton_color_pickers: List[ColorPicker] = all_skeleton_color_pickers
+        self.left_synchronizer_color_picker: ColorPicker = left_synchronizer_color_picker
+        self.right_synchronizer_color_picker: ColorPicker = right_synchronizer_color_picker
+        self.center_synchronizer_color_picker: ColorPicker = center_synchronizer_color_picker
 
     def sync_left_color_pickers(self, current_color):
-        for color_picker in self.left_color_pickers:
+        for color_picker in self.left_skeleton_color_pickers:
             color_picker.value = current_color.new
 
     def sync_right_color_pickers(self, current_color):
-        for color_picker in self.right_color_pickers:
+        for color_picker in self.right_skeleton_color_pickers:
             color_picker.value = current_color.new
 
     def sync_center_color_pickers(self, current_color):
-        for color_picker in self.center_color_pickers:
+        for color_picker in self.center_skeleton_color_pickers:
             color_picker.value = current_color.new
 
     def sync_all_color_pickers(self, current_color):
-        for color_picker in self.all_color_pickers:
+        for color_picker in self.all_skeleton_color_pickers:
             color_picker.value = current_color.new
+        self.left_synchronizer_color_picker.value = current_color.new
+        self.right_synchronizer_color_picker.value = current_color.new
+        self.center_synchronizer_color_picker.value = current_color.new
+
+
+class ColorChanger:
+    def __init__(self):
+        super(ColorChanger, self).__init__()
+
+    def get_color_changer_widget(self, skeletons: List[DrawableSkeleton]) -> Tab:
+        skeleton_color_changer_tabs: List[VBox] = []
+        pred_color_pickers: List[Tuple[ColorPicker, ColorPicker, ColorPicker, ColorPicker]] = []
+        gt_color_pickers: List[Tuple[ColorPicker, ColorPicker, ColorPicker, ColorPicker]] = []
+        for skeleton in skeletons:
+            skeleton_color_changer_tab, color_pickers = self.__create_color_changer_tab(skeleton)
+            skeleton_color_changer_tabs.append(skeleton_color_changer_tab)
+            if skeleton.is_ground_truth:
+                gt_color_pickers.append(color_pickers)
+            else:
+                pred_color_pickers.append(color_pickers)
+        if len(pred_color_pickers) != 0 and len(gt_color_pickers) != 0:
+            pred_skeleton_color_changer_tab = self.__create_color_synchronizer_tab(pred_color_pickers)
+            gt_skeleton_color_changer_tab = self.__create_color_synchronizer_tab(gt_color_pickers)
+            skeleton_color_changer_tabs.append(pred_skeleton_color_changer_tab)
+            skeleton_color_changer_tabs.append(gt_skeleton_color_changer_tab)
+        color_changer_widget: Tab = Tab(children=skeleton_color_changer_tabs)
+        for i in range(len(skeletons)):
+            color_changer_widget.set_title(i, 'Skeleton {:d} colors'.format(i + 1))
+        if len(skeleton_color_changer_tabs) > len(skeletons):
+            color_changer_widget.set_title(len(skeletons), 'Pred skeleton colors')
+            color_changer_widget.set_title(len(skeletons) + 1, 'GT skeleton colors')
+        return color_changer_widget
+
+    def __create_color_changer_tab(self, skeleton: DrawableSkeleton) ->\
+            Tuple[VBox, Tuple[ColorPicker, ColorPicker, ColorPicker, ColorPicker]]:
+        left_color_picker: ColorPicker = ColorPicker(
+            description='Left color:',
+            value=self.__get_as_html_color(skeleton.get_left_objects()[0].color))
+        right_color_picker: ColorPicker = ColorPicker(
+            description='Right color:',
+            value=self.__get_as_html_color(skeleton.get_right_objects()[0].color))
+        center_color_picker: ColorPicker = ColorPicker(
+            description='Center color:',
+            value=self.__get_as_html_color(skeleton.get_center_objects()[0].color))
+        all_color_picker: ColorPicker = ColorPicker(description='All color:', value='#ffffff')
+        skeleton_color_updater: SkeletonColorUpdater = SkeletonColorUpdater(
+            skeleton, left_color_picker, right_color_picker, center_color_picker)
+        left_color_picker.observe(skeleton_color_updater.update_left_objects, names='value')
+        right_color_picker.observe(skeleton_color_updater.update_right_objects, names='value')
+        center_color_picker.observe(skeleton_color_updater.update_center_objects, names='value')
+        all_color_picker.observe(skeleton_color_updater.update_all_objects, names='value')
+        return VBox([left_color_picker, right_color_picker, center_color_picker, all_color_picker]),\
+            (left_color_picker, right_color_picker, center_color_picker, all_color_picker)
+
+    @staticmethod
+    def __create_color_synchronizer_tab(
+            color_pickers: List[Tuple[ColorPicker, ColorPicker, ColorPicker, ColorPicker]]) -> VBox:
+        left_color_picker: ColorPicker = ColorPicker(description='Left color:', value='#ffffff')
+        right_color_picker: ColorPicker = ColorPicker(description='Right color:', value='#ffffff')
+        center_color_picker: ColorPicker = ColorPicker(description='Center color:', value='#ffffff')
+        all_color_picker: ColorPicker = ColorPicker(description='All color:', value='#ffffff')
+        color_picker_synchronizer: ColorPickerSynchronizer = ColorPickerSynchronizer(
+            left_skeleton_color_pickers=[color_picker[0] for color_picker in color_pickers],
+            right_skeleton_color_pickers=[color_picker[1] for color_picker in color_pickers],
+            center_skeleton_color_pickers=[color_picker[2] for color_picker in color_pickers],
+            all_skeleton_color_pickers=[color_picker[3] for color_picker in color_pickers],
+            left_synchronizer_color_picker=left_color_picker,
+            right_synchronizer_color_picker=right_color_picker,
+            center_synchronizer_color_picker=center_color_picker)
+        left_color_picker.observe(color_picker_synchronizer.sync_left_color_pickers, names='value')
+        right_color_picker.observe(color_picker_synchronizer.sync_right_color_pickers, names='value')
+        center_color_picker.observe(color_picker_synchronizer.sync_center_color_pickers, names='value')
+        all_color_picker.observe(color_picker_synchronizer.sync_all_color_pickers, names='value')
+        return VBox([left_color_picker, right_color_picker, center_color_picker, all_color_picker])
+
+    @staticmethod
+    def __get_as_html_color(color: int) -> str:
+        return '{0:#0{1}x}'.format(color, 8).replace('0x', '#')
 
 
 class SkeletonVisualizer:
@@ -110,8 +209,12 @@ class SkeletonVisualizer:
         self.__create_skeleton_plot(skeletons=skeletons, skeleton_converter=skeleton_converter, colors=colors,
                                     automatic_camera_orientation=automatic_camera_orientation, is_gt_list=is_gt_list)
         self.plot.display()
-        self.__display_checkboxes(include_names, include_coordinates)
-        self.__display_color_changer()
+        self.__link_text_widgets(include_names, include_coordinates)
+        visibility_widget: HBox = HBox([self.joint_names_visible, self.joint_coordinates_visible])
+        color_changer: ColorChanger = ColorChanger()
+        color_changer_tab: Tab = color_changer.get_color_changer_widget(self.skeletons)
+        self.__display_interface([('Change visibilities', visibility_widget),
+                                  ('Change colors', color_changer_tab)])
 
     def visualize_with_ground_truths(
             self, pred_skeletons: np.ndarray, gt_skeletons: np.ndarray,
@@ -147,10 +250,16 @@ class SkeletonVisualizer:
         self.__create_skeleton_plot(skeletons=first_frame, skeleton_converter=skeleton_converter, colors=colors,
                                     positions=skeleton_timestamps,
                                     automatic_camera_orientation=automatic_camera_orientation, is_gt_list=is_gt_list)
+        self.__link_text_widgets(include_names, include_coordinates)
         self.plot.display()
-        self.__display_video_player(fps, frames)
-        self.__display_checkboxes(include_names, include_coordinates)
-        self.__display_color_changer()
+        visibility_widget: HBox = HBox([self.joint_names_visible, self.joint_coordinates_visible])
+        video_player: VideoPlayer = VideoPlayer(self.plot)
+        video_player_widget: HBox = video_player.get_video_player_widget(fps, frames)
+        color_changer: ColorChanger = ColorChanger()
+        color_changer_tab: Tab = color_changer.get_color_changer_widget(self.skeletons)
+        self.__display_interface([('Change visibilities', visibility_widget),
+                                  ('Change colors', color_changer_tab),
+                                  ('Play video', video_player_widget)])
 
     def visualize_video_from_file(
             self, file_name: str, colors: List[Color] = None, fps: int = 15,
@@ -193,7 +302,7 @@ class SkeletonVisualizer:
     @staticmethod
     def __init_colors(number_of_skeletons: int, colors: List[Color]) -> List[Color]:
         if colors is None:
-            colors = ['default' for _ in range(number_of_skeletons)]
+            colors = ['default'] * number_of_skeletons
         else:
             assert number_of_skeletons == len(colors), \
                 'The \'skeletons\' and \'colors\' parameters must be the same length.'
@@ -266,29 +375,11 @@ class SkeletonVisualizer:
             self.plot += drawable_skeleton
             self.skeletons.append(drawable_skeleton)
 
-    def __display_video_player(self, fps: int, frames: np.ndarray) -> None:
-        video_player: Play = Play(
-            value=0,
-            min=0, max=frames.shape[0] - 1,
-            step=1, interval=1000 / fps,
-            description='Press play', disabled=False)
-        frame_updater = FrameUpdater(video_player)
-        frame_slider = IntSlider(value=0, min=0, max=frames.shape[0] - 1, step=1, description='Frame')
-        next_frame_button = Button(description='Next frame')
-        next_frame_button.on_click(frame_updater.update_to_next_frame)
-        previous_frame_button = Button(description='Previous frame')
-        previous_frame_button.on_click(frame_updater.update_to_previous_frame)
-        widgets.jslink((video_player, 'value'), (frame_slider, 'value'))
-        widgets.jslink((video_player, 'value'), (self.plot, 'time'))
-        display(widgets.HBox([video_player, previous_frame_button, frame_slider, next_frame_button]))
-
-    def __display_checkboxes(self, include_names: bool, include_coordinates: bool) -> None:
+    def __link_text_widgets(self, include_names: bool, include_coordinates: bool) -> None:
         if include_names:
             self.__link_joint_name_visibility_with_checkbox()
-            display(self.joint_names_visible)
         if include_coordinates:
             self.__link_joint_coordinate_visibility_with_checkbox()
-            display(self.joint_coordinates_visible)
 
     def __link_joint_name_visibility_with_checkbox(self) -> None:
         for skeleton in self.skeletons:
@@ -300,70 +391,12 @@ class SkeletonVisualizer:
             for joint_coordinate in skeleton.joint_coordinates:
                 widgets.jslink((joint_coordinate, 'visible'), (self.joint_coordinates_visible, 'value'))
 
-    def __display_color_changer(self):
-        skeleton_color_changer_tabs: List[VBox] = []
-        pred_color_pickers: List[Tuple[ColorPicker, ColorPicker, ColorPicker, ColorPicker]] = []
-        gt_color_pickers: List[Tuple[ColorPicker, ColorPicker, ColorPicker, ColorPicker]] = []
-        for skeleton in self.skeletons:
-            skeleton_color_changer_tab, color_pickers = self.__create_color_changer_tab(skeleton)
-            skeleton_color_changer_tabs.append(skeleton_color_changer_tab)
-            if skeleton.is_ground_truth:
-                gt_color_pickers.append(color_pickers)
-            else:
-                pred_color_pickers.append(color_pickers)
-        if len(pred_color_pickers) != 0 and len(gt_color_pickers) != 0:
-            pred_skeleton_color_changer_tab = self.__create_color_synchronizer_tab(pred_color_pickers)
-            gt_skeleton_color_changer_tab = self.__create_color_synchronizer_tab(gt_color_pickers)
-            skeleton_color_changer_tabs.append(pred_skeleton_color_changer_tab)
-            skeleton_color_changer_tabs.append(gt_skeleton_color_changer_tab)
-        color_changer_widget: Tab = Tab(children=skeleton_color_changer_tabs)
-        for i in range(len(self.skeletons)):
-            color_changer_widget.set_title(i, 'Skeleton {:d} colors'.format(i + 1))
-        if len(skeleton_color_changer_tabs) > len(self.skeletons):
-            color_changer_widget.set_title(len(self.skeletons), 'Pred skeleton colors')
-            color_changer_widget.set_title(len(self.skeletons) + 1, 'GT skeleton colors')
-        display(color_changer_widget)
-
-    def __create_color_changer_tab(self, skeleton: DrawableSkeleton) ->\
-            Tuple[VBox, Tuple[ColorPicker, ColorPicker, ColorPicker, ColorPicker]]:
-        left_color_picker: ColorPicker = ColorPicker(
-            description='Left color:', value=self.__get_as_html_color(skeleton.get_left_objects()[0].color))
-        right_color_picker: ColorPicker = ColorPicker(
-            description='Right color:', value=self.__get_as_html_color(skeleton.get_right_objects()[0].color))
-        center_color_picker: ColorPicker = ColorPicker(
-            description='Center color:', value=self.__get_as_html_color(skeleton.get_center_objects()[0].color))
-        all_color_picker: ColorPicker = ColorPicker(description='All color:', value='#ffffff')
-        skeleton_color_updater: SkeletonColorUpdater = SkeletonColorUpdater(
-            skeleton, left_color_picker, right_color_picker, center_color_picker)
-        left_color_picker.observe(skeleton_color_updater.update_left_objects, names='value')
-        right_color_picker.observe(skeleton_color_updater.update_right_objects, names='value')
-        center_color_picker.observe(skeleton_color_updater.update_center_objects, names='value')
-        all_color_picker.observe(skeleton_color_updater.update_all_objects, names='value')
-        return VBox([left_color_picker, right_color_picker, center_color_picker, all_color_picker]),\
-            (left_color_picker, right_color_picker, center_color_picker, all_color_picker)
-
     @staticmethod
-    def __create_color_synchronizer_tab(
-            color_pickers: List[Tuple[ColorPicker, ColorPicker, ColorPicker, ColorPicker]]) -> VBox:
-        left_color_picker: ColorPicker = ColorPicker(description='Left color:', value='#ffffff')
-        right_color_picker: ColorPicker = ColorPicker(description='Right color:', value='#ffffff')
-        center_color_picker: ColorPicker = ColorPicker(description='Center color:', value='#ffffff')
-        all_color_picker: ColorPicker = ColorPicker(description='All color:', value='#ffffff')
-        color_picker_synchronizer: ColorPickerSynchronizer = ColorPickerSynchronizer(
-            left_color_pickers=[color_picker[0] for color_picker in color_pickers],
-            right_color_pickers=[color_picker[1] for color_picker in color_pickers],
-            center_color_pickers=[color_picker[2] for color_picker in color_pickers],
-            all_color_pickers=[color_picker[3] for color_picker in color_pickers]
-        )
-        left_color_picker.observe(color_picker_synchronizer.sync_left_color_pickers, names='value')
-        right_color_picker.observe(color_picker_synchronizer.sync_right_color_pickers, names='value')
-        center_color_picker.observe(color_picker_synchronizer.sync_center_color_pickers, names='value')
-        all_color_picker.observe(color_picker_synchronizer.sync_all_color_pickers, names='value')
-        return VBox([left_color_picker, right_color_picker, center_color_picker, all_color_picker])
-
-    @staticmethod
-    def __get_as_html_color(color: int) -> str:
-        return '{0:#0{1}x}'.format(color, 8).replace('0x', '#')
+    def __display_interface(widget_tuples: List[Tuple[str, Widget]]) -> None:
+        interface: Accordion = Accordion(children=list([widget_tuple[1] for widget_tuple in widget_tuples]))
+        for i in range(len(widget_tuples)):
+            interface.set_title(i, widget_tuples[i][0])
+        display(interface)
 
 
 class MuPoTSVisualizer(SkeletonVisualizer):
